import java.lang.reflect.Array;
import java.util.Scanner;
import java.util.function.Consumer;

class GenericArray<T extends Comparable<T>> {
    private Class<? extends Comparable<?>> cls;
    private T[] arr;

    GenericArray(Class<? extends Comparable<?>> c) {
        cls = c;
        takeInput();
    }

    void takeInput() {
        Scanner sc = new Scanner(System.in);
        System.out.print("Enter the length: ");
        int size = sc.nextInt();
        arr = (T[]) Array.newInstance(cls, size);
        System.out.print("Enter the array elements: ");
        for (int i = 0; i < size; i++)
            arr[i] = parseElement(sc.next());
    }

    private T parseElement(String element) {
        try {
            if (cls == Character.class) return (T) Character.valueOf(element.charAt(0));
            return (T) cls.getConstructor(String.class).newInstance(element);
        } catch (Exception e) {
            throw new IllegalArgumentException("Error parsing element: " + element);
        }
    }

    void display() {
        System.out.print("The array elements are:");
        for (int i = 0; i < arr.length; i++)
            System.out.print(" " + arr[i]);
        System.out.println();
    }

    T get(int i)
            throws IndexOutOfBoundsException {
        return arr[i];
    }

    void set(int i, T val)
            throws IndexOutOfBoundsException {
        arr[i] = val;
    }

    int size() {
        return arr.length;
    }
}

class SortingMethods {
    static <T extends Comparable<T>> void bubbleSort(GenericArray<T> arr) {
        int n = arr.size();
        for (int i = 0; i < n - 1; i++) {
            for (int j = 0; j < n - i - 1; j++) {
                T x = arr.get(j), xp = arr.get(j + 1);
                if (x.compareTo(xp) > 0) {
                    arr.set(j, xp);
                    arr.set(j + 1, x);
                }
            }
        }
    }
    static <T extends Comparable<T>> void selectionSort(GenericArray<T> arr) {
        int n = arr.size();
        for (int i = 0; i < n - 1; i++) {
            int minidx = i;
            for (int j = i + 1; j < n; j++)
                if (arr.get(j).compareTo(arr.get(minidx)) < 0)
                    minidx = j;
            T tmp = arr.get(minidx);
            arr.set(minidx, arr.get(i));
            arr.set(i, tmp);
        }
    }
    static <T extends Comparable<T>> void insertionSort(GenericArray<T> arr) {
        int n = arr.size();
        for (int i = 1; i < n; i++) {
            T key = arr.get(i);
            int j;
            for (j = i - 1; j >= 0 && arr.get(j).compareTo(key) > 0; j--)
                arr.set(j + 1, arr.get(j));
            arr.set(j + 1, key);
        }
    }
}

class ArraySortGeneric {
    static Class primitiveToWrapperClass(String name) {
        switch (name) {
            case "int":
                return Integer.class;
            case "double":
                return Double.class;
            case "long":
                return Long.class;
            case "short":
                return Short.class;
            case "byte":
                return Byte.class;
            case "float":
                return Float.class;
            case "boolean":
                return Boolean.class;
            case "char":
                return Character.class;
            default:
                return null;
        }
    }

    static Class resolveClassFromName(String name)
            throws ClassNotFoundException {
        Class cls = primitiveToWrapperClass(name);
        if (cls != null) return cls;
        try {
            return Class.forName(name);
        } catch (ClassNotFoundException e) {
            if (!name.contains("."))
                return Class.forName("java.lang." + name);
            throw e;
        }
    }

    static void menu() {
        System.out.println(
            "Menu:\n" +
            " 1. (Re)enter array elements\n" +
            " 2. Display array elements\n" +
            " 3. Perform bubble sort\n" +
            " 4. Perform selection sort\n" +
            " 5. Perform insertion sort\n" +
            " 6. Exit\n");
    }

    static GenericArray takeArrayInput(Scanner sc) {
        System.out.print("Enter the type (class name) of the elements of the array: ");
        String name = sc.nextLine();
        try {
            Class<? extends Comparable<?>> cls = resolveClassFromName(name);
            GenericArray<?> arr = new GenericArray<>(cls);
            arr.display();
            return arr;
        } catch (ClassNotFoundException e) {
            System.out.println("Class '" + name + "' not found. Falling back to String.");
            return new GenericArray<>(String.class);
        }
    }

    static void performSort(GenericArray<?> arr, Consumer<GenericArray<?>> sorter) {
        System.out.print("Before sorting: ");
        arr.display();
        sorter.accept(arr);
        System.out.print("After sorting: ");
        arr.display();
    }

    public static void main(String[] args) {
        Scanner sc = new Scanner(System.in);
        System.out.println("Menu-driven program for sorting array\n");
        System.out.println("Enter the array:");
        GenericArray<?> arr = takeArrayInput(sc);
        while (true) {
            menu();
            System.out.print("Enter your choice: ");
            int choice = sc.nextInt();
            sc.skip("\n");
            switch (choice) {
                case 1:
                    arr = takeArrayInput(sc);
                    break;
                case 2:
                    arr.display();
                    break;
                case 3:
                    performSort(arr, SortingMethods::bubbleSort);
                    break;
                case 4:
                    performSort(arr, SortingMethods::selectionSort);
                    break;
                case 5:
                    performSort(arr, SortingMethods::insertionSort);
                    break;
                case 6:
                    System.out.println("Bye.");
                    return;
            }
        }
    }
}

