import java.util.*;
import java.util.stream.*;

interface Searcher<E> {
    Searcher<E> takeList(List<E> list);
    int search(E element);
}

class BinarySearcher<E extends Comparable<E>> implements Searcher<E> {
    List<E> list;
    Comparator<? super E> cmp = Comparator.<E>naturalOrder();
    public BinarySearcher() {}
    public BinarySearcher(Comparator<? super E> cmp) {
        this.cmp = cmp;
    }
    public Searcher<E> takeList(List<E> list) {
        this.list = list;
        this.list.sort(this.cmp);
        return this;
    }
    public int search(E element) {
        int size = list.size();
        int leftIndex = 0, rightIndex = size - 1;
        while (leftIndex <= rightIndex) {
            int midIndex = (leftIndex + rightIndex) / 2;
            E midValue = list.get(midIndex);
            int res = this.cmp.compare(element, midValue);
            if (res == 0) {
                return midIndex;
            } else if (res < 0) {
                rightIndex = midIndex - 1;
            } else {
                leftIndex = midIndex + 1;
            }
        }
        return -1;
    }
}

class PerformBinarySearch {
    public static void main(String args[]) {
        if (args.length == 0) {
            System.err.println("Usage: PerformBinarySearch ITEM ARRAY...");
            return;
        }
        List<Double> dlist = Arrays.asList(args).stream()
            .map(Double::parseDouble)
            .collect(Collectors.toList());
        double elementToFind = dlist.get(0);
        List<Double> list = dlist.subList(1, dlist.size());
        Searcher<Double> bs = new BinarySearcher<Double>().takeList(list);
        System.out.print("The given array:");
        for (int i = 0, len = list.size(); i < len; i++) {
            System.out.print(" " + list.get(i));
        }
        System.out.println();
        int pos = bs.search(elementToFind);
        if (pos == -1) {
            System.out.println("Element " + elementToFind + " could not be found in the given array.");
        } else {
            System.out.println("Element " + elementToFind + " was found in the given array at position " + (pos + 1) + ".");
        }
    }
}

