#import "tpl.typ": *
#show: body => apply(body)
#show heading: set text(font: "New Computer Modern Sans", weight: 450)
#let semibold(body) = text(weight: 600, body)
#text(size: 1.3em, font: "New Computer Modern Sans", align(center)[#semibold[Practical Assignments] \ _on_ \ #semibold[Programming in Python Lab]])
#set enum(full: true, numbering: (..args) => {
  let patterns = ("1.", "(a)")
  let pattern = patterns.at(calc.min(args.pos().len(), patterns.len()) - 1)
  numbering(pattern, args.pos().last())
})

#set raw(lang: "python")
#set par(justify: true)
#set heading(numbering: "1 ")

= Basic Python Syntax
#v(1em)

+ Use the Python prompt as a basic calculator, explore the order of operations using parentheses.
+ Explore the various functions in the math module. For example: find the GCD of two numbers, area and perimeter of circle using `math.pi` etc.
+ Exploring the complex data type and their operations, For example: finding the modulus and phase angle of a complex number.
+ The print function – Printing values. Repeat the previous experiments now using the print function.
+ Write a simple python script using the print function in a text editor, save it with the extension `.py`. Run it in the terminal/command prompt.
+ Write a simple python program to ask the user to enter two numbers and output the sum, product, difference and the GCD.
+ Write a python program to explore the `re` module, especially `re.split`, `re.join`, `re.search` and `re.match` methods.
+ Write a program to take values of different datatypes as input from the user and display them.
+ Write a program to take two floating point numbers as input from the user. Concatenate the integral parts of the two numbers and display them, also display the sum of the input floating point numbers rounded upto 2 decimal places.
+ Write a program to swap the contents of two variables without using any auxillary variable.
+ Write a program to divide two numbers and check if the digits at one’s place of the quotient and remainder are equal.
+ Write a program to take the number of coins each of value Rs 10, Rs 5, Rs 2 and Rs 1 from the user and display the total amount of money.

= Decision Control Statements
#v(1em)

+ Use while loops to do Gaussian addition on a list having an even number of numbers. Print each partial sum. For example: if the list is `[1, 2, 3, 4, 5, 6]`, the program should output `1 + 6`, `2 + 5`, and `3 + 4` in separate lines, and the result of the addition `21`. Extend it to handle lists of odd length.
+ Primality testing using for and while loops.
+ Use the previous assignment to generate a list of primes within a user-given range.
+ Explore the `key` function of `sum()`, `min()`, `max()`, and `sort()` functions using `lambda`s.
+ Write a simple game in Python. A random number between 1 and 10 (1 and 10 inclusive) is selected as a secret number, then the user is prompted to guess the number. If the user’s guess matches the secret number, the game ends with an appropriate message, otherwise the user is asked to try again. After 3 unsuccessful guesses, the user loses and the game ends with an appropriate message revealing the secret number. Include a provision at the end of the game to ask the user to play again. (Use the random module to obtain the secret number.)
+ Write a program to generate the calendar of a month given the start day (1-7) and the number of days in the month. The start day should be taken as a integer where `1` is Sunday, `2` is Monday, `3` is Tuesday, ... `7` is Saturday.
+ Write a program to read numbers from the user until `-1` is encountered. Print the number of prime numbers and the number of composite numbers entered by the user.
+ Write a program that counts the number of lowercase characters, uppercase characters and digits entered by the user. Take input as a single string.
+ #table(
    columns: (1fr, 4em), stroke: none,
    inset: 0pt, gutter: 0pt,
    column-gutter: 5pt,
    [Write a program to print the following pattern shown for number of lines = 4. The program must be able to generate the same pattern for any arbitrary number of lines.],
    [
      #set par(leading: 0em)
      #set block(inset: 0pt, fill: white, width: 4em, outset: 0pt)
      ```text
****
*  *
*  *
****
```
  ])

= Strings
#v(1em)

+ Write a python program to ask the user for two strings, print a new string where the first string is reversed, and the second string is converted to upper case. Sample strings: `Pets`, `party`, output: `steP PARTY`. Only use string slicing and `+` operator.
+ Write a python program to from a list of words, join all the words in the odd and even indices to form two strings. Use list slicing and join methods.
+ Write a program to input your name in a string and separate the first name and last name using the slice operator with negative indexing. Display your name in reverse using a formatted string.
+ Write a program to simulate the functionality of the `count()` method.
+ Write a program to take input a string and display it after removing all the vowels.
+ Write a program that encrypts a message by adding a key value to every character. (Also known as _Caesar Cipher_.)
+ Write a program to separate the username and domain name of an email address given as per the RFC 5322 format.

= Lists
#v(1em)

+ Write a python program to simulate a stack and a queue using lists. Note that the queue deletion operation won’t run in $O(1)$ time.
+ Use list comprehension to find all the odd numbers and numbers divisible by 3 from a list of numbers.
+ Write a program to find the second largest number in a list of numbers.
+ Use list comprehensions to generate a list of numbers items as per the following condition where $i$ denotes the index of the list:
  $ "items"[ i ] = cases(
    i^2 quad "if " i "is even",
    i^3 quad "if " i "is odd",
  )  $
+ Write a program to use the `map()` method to square each number in a list
+ Write a program to remove all duplicate items in a list.

= Tuples
#v(1em)

+ Write a program that has two sequences. The first sequence stores first names, and the second sequence stores last names. Use the `zip()` function to generate a tuple of tuples each of which contains the full name of a person.
+ Create a tuple and modify it using various techniques.
+ Use a nested tuple to store the Name, Roll No. and Marks of some students and find the name of the student having highest marks.
+ Write a program that has a list of numbers (both positive and negative). Make a new tuple out of this list that has only positive values.
+ Write a function to implement variable length arguments (scatter and gather) using the `*` symbol.

= Sets
#v(1em)

+ Use sets to de-duplicate a list of numbers and a string such that they contain only the unique elements.
+ Use the set union and intersection operations to implement the Jaccard and cosine similarity of two sets.
+ Write a program to observe the difference between re`move()` and `discard()` methods.
4. Write a program that creates two sets. One has even numbers in the range 1-10 and the other has all composite numbers in the range 1-20. Demonstrate the use of `all()`, `issuperset()`, `len()`, `sum()` functions on the sets.
+ Write a program that generates a set of prime numbers and another set of odd numbers. Demonstrate the result of union, intersection, difference, symmetric difference operations on these sets.
+ Write a program that creates two sets - squares and cubes in the range 1-10. Demonstrate the use of `update()`, `pop()`, `remove()`, `add()` and `clear()` functions.
+ Demonstrate the use of frozensets.

= Dictionaries
#v(1em)

+ Use dictionaries to count the word and letter occurrences in a long string of text.
+ Invert a dictionary such the previous keys become values and values become keys. For example: if the initial and inverted dictionaries are `d1` and `d2`, where `d1 = {1: 'a', 2: 'b', 3: 120}`, then `d2 = {'a': 1, 2: 'b', 120: 3}`.
+ What if the values in 2 are not immutable? Use frozensets, For repeated values, use lists. For example: if `d1 = {1: 'a', 2: 'a', 4: [1, 2]}`, then `d2 = {'a': [1, 2], frozenset([1, 2]): 4}`.
+ Write a function to generate the Fibonacci numbers in
  + Exponential time using the naïve algorithm.
  + Linear time using dynamic programming (memorization) with a dictionary.
+ Use dictionary comprehensions to generate a dictionary where each key is a number and the corresponding value is the number’s square.
+ Write a program to store sparse matrix as a dictionary.
+ Write a program that combines two lists to a dictionary.

= User-Defined Functions
#v(1em)

+ Implement popular sorting algorithms like quick sort and merge sort to sort lists of numbers.
+ Implement the Pascal’s triangle.
+ Three positive integers a, b, and c are Pythagorean triples if $a^2 + b^2 = c^2$. Write a function to generate all Pythagorean triples in a certain range.
+ Write two functions that simulate the toss of a fair coin, and the roll of an unbiased `n` sided die using the random module.
+ Modify the previous assignment. Now the coin and the die are not fair, with each outcome having a given probability.

= File Handling; `sys`, `pickle` and `csv` modules

+ Basic file operations. Explore the different file modes.
+ Emulate the unix `cp`, `grep`, `cat` programs in Python. In each case, the user should pass the arguments to the program as command line arguments.
+ Use pickle for persistent storage of variables.

= Object Oriented Programming
#v(1em)

+ Create a `Graph` class to store and manipulate graphs. It should have the following functions:
  + Read an edge list file, where each edge `(u, v)` appears exactly once in the file as space separated values.
  + Add and remove nodes and edges.
  + Print nodes, and edges in a user readable format.
  + Computes basic statistics of the graph like degree distribution, clustering coefficient, and the number of connected components.
  + Finding all the neighbors of a node.
  + Finding all the connected components and storing them as individual Graph objects inside the class.
  + Finding single source shortest paths using Breadth First Search.
+ Make a `DiGraph` class to handle directed graphs which inherits from the `Graph` class. In addition to all of the functionalities of 1, it should support the following operations
  + Finding the predecessors and successors of a node.
  + Creating a new `DiGraph` object where all the edges are reversed.
  + Finding the strongly connected components.
+ Extend 1 and 2 to handle weighted graphs, and implement Dijkstra’s and Floyd-Warshall algorithms to compute the single source and all pairs shortest paths.
+ Use the graph containers in 1, 2, and 3 to implement additional graph algorithms.


